# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
"""

Class for storing hawkauth nonces in memcached.

"""

import time
import math
from hashlib import sha1
from base64 import urlsafe_b64encode

from mozsvc.storage.mcclient import MemcachedClient


DEFAULT_TIMESTAMP_WINDOW = 60


class MemcachedNonceCache(object):
    """Object for managing a cache of used nonce values in memcached.

    This class allow easy timestamp-based management of client-generated
    nonces for hawkauthlib.

    It stores the nonces in memcached so that they can be shared between
    different webserver processes.  Each timestamp+nonce combo is stored
    under a key sha1(<timestamp>:<nonce>).
    """

    def __init__(self, window=None, get_time=None, cache_server=None,
                 cache_key_prefix="noncecache:", cache_pool_size=None,
                 cache_pool_timeout=60, **kwds):
        # Memcached ttls are in integer seconds, so round up to the nearest.
        if window is None:
            window = DEFAULT_TIMESTAMP_WINDOW
        else:
            window = int(math.ceil(window))
        self.window = window
        self.get_time = get_time or time.time
        self.mcclient = MemcachedClient(cache_server, cache_key_prefix,
                                        cache_pool_size, cache_pool_timeout)

    def __len__(self):
        raise NotImplementedError

    def check_nonce(self, timestamp, nonce):
        """Check if the given timestamp+nonce is fresh.

        This method checks that the given timestamp is within the configured
        time window, and that the given nonce has not previously been seen
        with that timestamp.  It returns True if the nonce is fresh and False
        if it is stale.

        Fresh nonces are stored in memcache so that subsequent checks of the
        same nonce will return False.
        """
        now = self.get_time()
        # Check if the timestamp is within the configured window.
        ts_min = now - self.window
        ts_max = now + self.window
        if not ts_min < timestamp < ts_max:
            return False
        # Check if it's in memcached, adding it if not.
        # Fortunately memcached 'add' has precisely the right semantics
        # of "create if not exists"
        key = urlsafe_b64encode(sha1("%d:%s" % (timestamp, nonce)).digest())
        try:
            if not self.mcclient.add(key, 1, time=self.window):
                return False
        except ValueError:
            return False
        # Successfully added, the nonce must be fresh.
        return True



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest

import pyramid.testing
from pyramid.response import Response

from mozsvc.exceptions import BackendError
from mozsvc.tests.support import make_request


class TestErrorHandlingTweens(unittest.TestCase):

    def setUp(self):
        self.app = None
        self.config = pyramid.testing.setUp()
        self.config.registry.settings["mozsvc.retry_after"] = "17"
        self.config.include("mozsvc")
        self.config.add_route("backend_error", "/backend_error")

    def tearDown(self):
        pyramid.testing.tearDown()

    def _make_request(self, *args, **kwds):
        return make_request(self.config, *args, **kwds)

    def _do_request(self, *args, **kwds):
        if self.app is None:
            self.app = self.config.make_wsgi_app()
        req = self._make_request(*args, **kwds)
        return self.app.handle_request(req)

    def _set_backend_error_view(self, func):
        self.config.add_view(func, route_name="backend_error")

    def test_that_backend_errors_are_captured(self):
        @self._set_backend_error_view
        def backend_error(request):
            raise BackendError

        r = self._do_request("/backend_error")
        self.assertEquals(r.status_int, 503)
        retry_after = int(r.headers["Retry-After"])
        self.assertTrue(17 <= retry_after <= 22)

    def test_that_backend_errors_can_set_retry_after(self):
        @self._set_backend_error_view
        def backend_error(request):
            raise BackendError(retry_after=42)

        r = self._do_request("/backend_error")
        self.assertEquals(r.status_int, 503)
        retry_after = int(r.headers["Retry-After"])
        self.assertTrue(42 <= retry_after <= 47)

    def test_that_retry_after_doesnt_get_set_to_zero(self):
        @self._set_backend_error_view
        def backend_error(request):
            raise BackendError(retry_after=0)

        r = self._do_request("/backend_error")
        self.assertEquals(r.status_int, 503)
        self.assertEquals(r.headers.get("Retry-After"), None)


class TestBackoffResponseTween(unittest.TestCase):

    def setUp(self):
        self.app = None
        self.config = pyramid.testing.setUp()
        self.config.add_route("root", "/")
        self.config.add_view(lambda r: Response("ok"), route_name="root")

    def tearDown(self):
        pyramid.testing.tearDown()

    def _make_request(self, *args, **kwds):
        return make_request(self.config, *args, **kwds)

    def _do_request(self, *args, **kwds):
        if self.app is None:
            self.app = self.config.make_wsgi_app()
        req = self._make_request(*args, **kwds)
        return self.app.handle_request(req)

    def _do_requests(self, count=100):
        # Do some requests and return the total number performed, the number
        # that had backoff headers, and the number that had unavailable errors.
        backoff_count = 0
        unavail_count = 0
        for _ in xrange(count):
            r = self._do_request("/")
            self.assertTrue(r.status_int in (200, 503))
            if r.status_int == 503:
                unavail_count += 1
            if "X-Backoff" in r.headers:
                backoff_count += 1
        return count, backoff_count, unavail_count

    def test_that_backoff_responses_are_not_sent_by_default(self):
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertEquals(backoff_count, 0)
        self.assertEquals(unavail_count, 0)

    def test_that_backoff_headers_can_be_sent_uniformly(self):
        self.config.registry.settings["mozsvc.backoff_probability"] = 1
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertEquals(backoff_count, count)
        self.assertEquals(unavail_count, 0)

    def test_that_backoff_headers_can_be_sent_probabilistically(self):
        self.config.registry.settings["mozsvc.backoff_probability"] = 0.5
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertTrue(backoff_count > 0)
        self.assertTrue(backoff_count < count)
        self.assertEquals(unavail_count, 0)

    def test_that_unavail_responses_can_be_sent_uniformly(self):
        self.config.registry.settings["mozsvc.unavailable_probability"] = 1
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertEquals(backoff_count, 0)
        self.assertEquals(unavail_count, count)

    def test_that_unavail_responses_can_be_sent_probabilistically(self):
        self.config.registry.settings["mozsvc.unavailable_probability"] = 0.5
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertEquals(backoff_count, 0)
        self.assertTrue(unavail_count > 0)
        self.assertTrue(unavail_count < count)

    def test_that_unavail_and_backoff_can_be_used_together(self):
        self.config.registry.settings["mozsvc.backoff_probability"] = 0.5
        self.config.registry.settings["mozsvc.unavailable_probability"] = 0.5
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertTrue(backoff_count > 0)
        self.assertTrue(backoff_count < count)
        self.assertTrue(unavail_count > 0)
        self.assertTrue(unavail_count < count)

    def test_that_unavail_and_backoff_can_be_passed_as_strings(self):
        self.config.registry.settings["mozsvc.backoff_probability"] = "0.5"
        self.config.registry.settings["mozsvc.unavailable_probability"] = "0.5"
        self.config.include("mozsvc")
        count, backoff_count, unavail_count = self._do_requests()
        self.assertTrue(backoff_count > 0)
        self.assertTrue(backoff_count < count)
        self.assertTrue(unavail_count > 0)
        self.assertTrue(unavail_count < count)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
"""

Hawk nonce-checking class that doesn't actually check them.

"""

import time
import logging


logger = logging.getLogger("mozsvc.user")


class PermissiveNonceCache(object):
    """Object for not really managing a cache of used nonce values.

    This class implements the timestamp/nonce checking interface required
    by hawkauthlib, but doesn't actually check them.  Instead it just logs
    timestamps that are too far out of the timestamp window for future
    analysis.
    """

    def __init__(self, log_window=60, get_time=None):
        self.log_window = log_window
        self.get_time = get_time or time.time

    def __len__(self):
        raise NotImplementedError

    def check_nonce(self, timestamp, nonce):
        """Check if the given timestamp+nonce is fresh."""
        now = self.get_time()
        skew = now - timestamp
        if abs(skew) > self.log_window:
            logger.warn("Large timestamp skew detected: %d", skew)
        return True



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import time
import unittest2
import tempfile

from zope.interface import implements

import pyramid.testing
import pyramid.request
from pyramid.interfaces import IAuthenticationPolicy
from pyramid.httpexceptions import HTTPUnauthorized

import tokenlib
import hawkauthlib

from mozsvc.exceptions import BackendError
from mozsvc.tests.support import TestCase
from mozsvc.secrets import DerivedSecrets
from mozsvc.user.permissivenoncecache import PermissiveNonceCache
from mozsvc.user import TokenServerAuthenticationPolicy

try:
    from mozsvc.storage.mcclient import MemcachedClient
    from mozsvc.user.noncecache import MemcachedNonceCache
    # We'll test for a live memcached server when we actually run the tests.
    MEMCACHED = None
except (ImportError, BackendError):
    MEMCACHED = False


class ExpandoRequest(object):
    """Proxy class for setting arbitrary attributes on a request.

    This class allows us to easily stub out various derived properties of
    a Request object.  Mostly notable, Request.host_url which is usually
    a read-only property derviced from the request evnironment.
    """

    def __init__(self, req):
        self.__dict__["_ExpandoRequest__req"] = req

    def __getattr__(self, attr):
        return getattr(self.__req, attr)

    def __setattr__(self, attr, value):
        try:
            setattr(self.__req, attr, value)
        except AttributeError:
            self.__dict__[attr] = value


class StubAuthenticationPolicy(object):
    """Authentication policy taking creds from request headers.

    This is a stub authentication policy that takes username from the
    X-Username header, checks that it matches the password provided in
    the X-Password header, and returns that as the userid.
    """

    implements(IAuthenticationPolicy)

    def authenticated_userid(self, request):
        username = request.environ.get("HTTP_X_USERNAME")
        if not username:
            return None
        password = request.environ.get("HTTP_X_PASSWORD")
        if not password:
            return None
        # Don't panic!  This is only used in the tests as an easy way
        # to simulate both failed and successfull logins.
        if password != username:
            return None
        request.user["x-was-ere"] = True
        return username

    def unauthenticated_userid(self, request):
        raise RuntimeError("tests shouldn't call this")  # pragma: nocover

    def effective_principals(self, request):
        raise RuntimeError("tests shouldn't call this")  # pragma: nocover


class UserTestCase(TestCase):

    def setUp(self):
        super(UserTestCase, self).setUp()
        self.config.commit()
        self.policy = self.config.registry.queryUtility(IAuthenticationPolicy)

    def get_configurator(self):
        config = super(UserTestCase, self).get_configurator()
        config.include("mozsvc.user")
        return config

    def test_that_the_correct_policy_defaults_are_used(self):
        policy = self.policy
        self.assertTrue(isinstance(policy, TokenServerAuthenticationPolicy))
        self.assertTrue(isinstance(policy.nonce_cache, PermissiveNonceCache))
        self.assertTrue(policy.secrets is None)

    def test_that_authn_policy_can_be_overridden(self):
        self.config.set_authentication_policy(StubAuthenticationPolicy())
        self.config.commit()
        # Good password => successful auth via request.user
        request = self.make_request(environ={
            "HTTP_X_USERNAME": "user1",
            "HTTP_X_PASSWORD": "user1",
        })
        self.assertEquals(request.user["uid"], "user1")
        self.assertEquals(request.user["x-was-ere"], True)
        # Bad password => request.user is empty
        request = self.make_request(environ={
            "HTTP_X_USERNAME": "user1",
            "HTTP_X_PASSWORD": "BAD PASSWORD",
        })
        self.assertFalse(request.user)
        # No username => request.user is empty
        request = self.make_request(environ={
            "HTTP_X_PASSWORD": "password1",
        })
        self.assertFalse(request.user)
        # No password => request.user is empty
        request = self.make_request(environ={
            "HTTP_X_USERNAME": "user1",
        })
        self.assertFalse(request.user)

    def test_that_hawkauth_is_used_by_default(self):
        # Generate signed request.
        req = self.make_request()
        tokenid, key = self.policy.encode_hawk_id(req, 42)
        hawkauthlib.sign_request(req, tokenid, key)
        # That should be enough to authenticate.
        self.assertEquals(req.authenticated_userid, 42)
        self.assertEquals(req.user.get("uid"), 42)
        # Check that it rejects invalid Hawk ids.
        req = self.make_request()
        hawkauthlib.sign_request(req, tokenid, key)
        authz = req.environ["HTTP_AUTHORIZATION"]
        req.environ["HTTP_AUTHORIZATION"] = authz.replace(tokenid, "XXXXXX")
        with self.assertRaises(HTTPUnauthorized):
            req.authenticated_userid
        # And that the rejection gets raised when accessing request.user
        self.assertRaises(HTTPUnauthorized, getattr, req, "user")

    def test_that_req_user_can_be_replaced(self):
        req = self.make_request()
        tokenid, key = self.policy.encode_hawk_id(req, 42)
        hawkauthlib.sign_request(req, tokenid, key)
        req.user = {"uid": 7}
        self.assertEquals(req.user, {"uid": 7})

    def test_that_hawkauth_cant_use_both_secret_and_secrets_file(self):
        config2 = pyramid.testing.setUp()
        config2.add_settings({
            "hawkauth.secret": "DARTH VADER IS LUKE'S FATHER",
            "hawkauth.secrets_file": "/dev/null",
        })
        self.assertRaises(ValueError, config2.include, "mozsvc.user")

    def test_that_hawkauth_can_use_custom_secrets_backend(self):
        config2 = pyramid.testing.setUp()
        config2.add_settings({
            "hawkauth.secrets.backend": "mozsvc.secrets.DerivedSecrets",
            "hawkauth.secrets.master_secrets": "abcd 123456",
        })
        config2.include("mozsvc.user")
        policy2 = config2.registry.queryUtility(IAuthenticationPolicy)
        self.assertTrue(isinstance(policy2.secrets, DerivedSecrets))

    def test_that_hawkauth_can_use_per_node_hostname_secrets(self):
        with tempfile.NamedTemporaryFile() as sf:
            # Write some secrets to a file.
            sf.write("http://host1.com,0001:secret11,0002:secret12\n")
            sf.write("https://host2.com,0001:secret21,0002:secret22\n")
            sf.write("https://host3.com:444,0001:secret31,0002:secret32\n")
            sf.flush()
            # Configure the plugin to load them.
            config2 = pyramid.testing.setUp()
            config2.add_settings({
                "hawkauth.secrets_file": sf.name,
            })
            config2.include("mozsvc.user")
            config2.commit()
            # It should accept a request signed with the old secret on host1.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host1.com",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret11")
            key = tokenlib.get_token_secret(id, secret="secret11")
            hawkauthlib.sign_request(req, id, key)
            self.assertEquals(req.authenticated_userid, 42)
            # It should accept a request signed with the new secret on host1.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host1.com",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret12")
            key = tokenlib.get_token_secret(id, secret="secret12")
            hawkauthlib.sign_request(req, id, key)
            self.assertEquals(req.authenticated_userid, 42)
            # It should reject a request signed with secret from other host.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host2.com",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret12")
            key = tokenlib.get_token_secret(id, secret="secret12")
            hawkauthlib.sign_request(req, id, key)
            with self.assertRaises(HTTPUnauthorized):
                req.authenticated_userid
            # It should reject a request over plain http when host2 is ssl.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host2.com",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret22")
            key = tokenlib.get_token_secret(id, secret="secret22")
            hawkauthlib.sign_request(req, id, key)
            with self.assertRaises(HTTPUnauthorized):
                req.authenticated_userid
            # It should accept a request signed with the new secret on host2.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host2.com",
                "wsgi.url_scheme": "https",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret22")
            key = tokenlib.get_token_secret(id, secret="secret22")
            hawkauthlib.sign_request(req, id, key)
            self.assertEquals(req.authenticated_userid, 42)
            # It should accept a request to host1 with an explicit port number.
            # Use some trickery to give host_url a value with default port.
            req = ExpandoRequest(self.make_request(config=config2, environ={
                "HTTP_HOST": "host1.com:80",
                "wsgi.url_scheme": "http",
            }))
            req.host_url = "http://host1.com:80"
            id = tokenlib.make_token({"uid": 42, "node": req.host_url[:-3]},
                                     secret="secret11")
            key = tokenlib.get_token_secret(id, secret="secret11")
            hawkauthlib.sign_request(req, id, key)
            self.assertEquals(req.authenticated_userid, 42)
            # It should accept a request to host2 with an explicit port number.
            # Use some trickery to give host_url a value with default port.
            req = ExpandoRequest(self.make_request(config=config2, environ={
                "HTTP_HOST": "host2.com:443",
                "wsgi.url_scheme": "https",
            }))
            req.host_url = "https://host2.com:443"
            id = tokenlib.make_token({"uid": 42, "node": req.host_url[:-4]},
                                     secret="secret22")
            key = tokenlib.get_token_secret(id, secret="secret22")
            hawkauthlib.sign_request(req, id, key)
            self.assertEquals(req.authenticated_userid, 42)
            # It should accept a request to host3 on a custom port.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host3.com:444",
                "wsgi.url_scheme": "https",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret32")
            key = tokenlib.get_token_secret(id, secret="secret32")
            hawkauthlib.sign_request(req, id, key)
            self.assertEquals(req.authenticated_userid, 42)
            # It should reject unknown hostnames.
            req = self.make_request(config=config2, environ={
                "HTTP_HOST": "host4.com",
            })
            id = tokenlib.make_token({"uid": 42, "node": req.host_url},
                                     secret="secret12")
            key = tokenlib.get_token_secret(id, secret="secret12")
            hawkauthlib.sign_request(req, id, key)
            with self.assertRaises(HTTPUnauthorized):
                req.authenticated_userid

    def test_checking_of_token_node_assignment(self):
        # Generate a token for one node
        req = self.make_request(environ={
            "HTTP_HOST": "host1.com",
        })
        tokenid, key = self.policy.encode_hawk_id(req, 42)
        # It can authenticate for requests to that node.
        hawkauthlib.sign_request(req, tokenid, key)
        self.assertEquals(req.authenticated_userid, 42)
        self.assertEquals(req.user.get("uid"), 42)
        # But not requests to some other node.
        req = self.make_request(environ={
            "HTTP_HOST": "host2.com",
        })
        hawkauthlib.sign_request(req, tokenid, key)
        with self.assertRaises(HTTPUnauthorized):
            req.authenticated_userid


class TestMemcachedNonceCache(unittest2.TestCase):

    def setUp(self):
        global MEMCACHED
        if MEMCACHED is None:
            try:
                MemcachedClient().get("")
            except BackendError:
                MEMCACHED = False
            else:
                MEMCACHED = True
        if not MEMCACHED:
            raise unittest2.SkipTest("no memcache")
        self.nc = None
        self.keys_to_delete = set()

    def tearDown(self):
        if self.nc is not None:
            for key in self.keys_to_delete:
                self.nc.mcclient.delete(key)

    def _monkeypatch_mcclient(self, mcclient):
        # Ultramemcache has no API for clearing all keys.
        # Monkeypatch it to remember any keys we use,
        # so we can delete them during cleanup.
        orig_add = mcclient.add

        def add_and_remember(key, *args, **kwds):
            self.keys_to_delete.add(key)
            return orig_add(key, *args, **kwds)

        mcclient.add = add_and_remember

    def test_operation(self, now=lambda: int(time.time())):
        window = 5
        nc = self.nc = MemcachedNonceCache(window=window)
        self._monkeypatch_mcclient(nc.mcclient)
        # Initially nothing is cached, so all nonces as fresh.
        ts = now()
        try:
            self.assertTrue(nc.check_nonce(ts, "abc"))
        except BackendError:
            raise unittest2.SkipTest("no memcache")
        # After that check, the (ts, nonce) pair should be stale.
        # Changing either the ts or the nonce will make it fresh.
        self.assertFalse(nc.check_nonce(ts, "abc"))
        self.assertTrue(nc.check_nonce(ts, "xyz"))
        self.assertTrue(nc.check_nonce(ts + 1, "abc"))
        # Timestamps outside the configured window are rejected.
        self.assertFalse(nc.check_nonce(now() - window - 1, "abc"))
        self.assertFalse(nc.check_nonce(now() + window + 1, "abc"))


class TestPermissiveNonceCache(unittest2.TestCase):

    def test_permissiveness(self):
        nc = PermissiveNonceCache()
        self.assertTrue(nc.check_nonce(time.time(), "abcd"))
        self.assertTrue(nc.check_nonce(1234, "abcd"))
        self.assertTrue(nc.check_nonce(1234, "abcd"))
        self.assertTrue(nc.check_nonce(987654321987654321, "hijk"))




import time
import unittest2

from pyramid.request import Request, Response
from pyramid.httpexceptions import HTTPNotFound, HTTPForbidden

from webtest import TestApp
from testfixtures import LogCapture
import pyramid.testing

from mozsvc.metrics import metrics_timer, initialize_request_metrics

from cornice import Service
from cornice.pyramidhook import register_service_views


class TestMetrics(unittest2.TestCase):

    def setUp(self):
        self.logs = LogCapture()

    def tearDown(self):
        self.logs.uninstall()

    def test_service_metrics(self):
        stub_service = Service(name="stub", path="/stub")

        @stub_service.get()
        @metrics_timer("view_time")
        def stub_view(request):
            request.metrics["stub"] = "stub-a-dub-dub"
            return {}

        with pyramid.testing.testConfig() as config:
            config.include("cornice")
            config.include("mozsvc")
            register_service_views(config, stub_service)
            app = TestApp(config.make_wsgi_app())
            res = app.get("/stub")
            self.assertEquals(res.body, "{}")

        self.assertTrue(len(self.logs.records), 1)
        r = self.logs.records[0]
        self.assertEquals(r.stub, "stub-a-dub-dub")
        self.assertTrue(0 < r.request_time < 0.1)
        self.assertTrue(0 < r.view_time <= r.request_time)

    def test_timing_decorator(self):

        @metrics_timer("timer1")
        def doit1():
            time.sleep(0.01)

        def viewit(request):
            doit1()

        request = Request.blank("/")
        initialize_request_metrics(request)
        with pyramid.testing.testConfig(request=request):
            viewit(request)

        ts = request.metrics["timer1"]
        self.assertTrue(0.01 < ts < 0.1)

    def test_timing_contextmanager(self):

        def viewit(request):
            with metrics_timer("timer1"):
                time.sleep(0.01)

        request = Request.blank("/")
        initialize_request_metrics(request)
        with pyramid.testing.testConfig(request=request):
            viewit(request)

        ts = request.metrics["timer1"]
        self.assertTrue(0.01 < ts < 0.1)

    def test_timing_contextmanager_with_explicit_request_object(self):

        def viewit(request):
            with metrics_timer("timer1", request):
                time.sleep(0.01)

        request = Request.blank("/")
        initialize_request_metrics(request)
        viewit(request)

        ts = request.metrics["timer1"]
        self.assertTrue(0.01 < ts < 0.1)

    def test_timing_contextmanager_doesnt_fail_if_no_metrics_dict(self):

        def viewit(request):
            with metrics_timer("timer1"):
                time.sleep(0.01)

        request = Request.blank("/")
        with pyramid.testing.testConfig(request=request):
            viewit(request)

        self.assertFalse(hasattr(request, "metrics"))

    def test_timing_contextmanager_doesnt_fail_if_no_reqest_object(self):
        with metrics_timer("timer1"):
            time.sleep(0.01)

    def test_that_service_metrics_include_correct_response_codes(self):
        stub_service = Service(name="stub", path="/{what}")

        @stub_service.get()
        def stub_view(request):
            what = request.matchdict["what"]
            if what == "ok":
                return Response(status=200)
            if what == "notfound":
                return Response(status=404)
            if what == "forbidden":
                return Response(status=403)
            if what == "exc_forbidden":
                raise HTTPForbidden
            if what == "impl_forbidden":
                request.response.status_code = 403
                return ""
            raise HTTPNotFound

        with pyramid.testing.testConfig() as config:
            config.include("cornice")
            config.include("mozsvc")
            register_service_views(config, stub_service)
            app = TestApp(config.make_wsgi_app())

            app.get("/ok", status=200)
            r = self.logs.records[-1]
            self.assertEquals(r.code, 200)

            app.get("/notfound", status=404)
            r = self.logs.records[-1]
            self.assertEquals(r.code, 404)
            app.get("/forbidden", status=403)
            r = self.logs.records[-1]
            self.assertEquals(r.code, 403)

            app.get("/exc_notfound", status=404)
            r = self.logs.records[-1]
            self.assertEquals(r.code, 404)
            app.get("/exc_forbidden", status=403)
            r = self.logs.records[-1]
            self.assertEquals(r.code, 403)

            app.get("/impl_forbidden", status=403)
            r = self.logs.records[-1]
            self.assertEquals(r.code, 403)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

import sys
import random
import traceback
import simplejson as json

from pyramid.httpexceptions import HTTPException, HTTPServiceUnavailable

import mozsvc
from mozsvc.util import safer_format_traceback
from mozsvc.exceptions import BackendError
from mozsvc.middlewares import create_hash


def catch_backend_errors(handler, registry):
    """Tween to turn BackendError into a 503 response.

    This is a pyramid tween factory for catching BackendError exceptions
    and translating them into a HTTP "503 Service Unavailable" response.
    """
    def catch_backend_errors_tween(request):
        try:
            return handler(request)
        except BackendError as err:
            err_info = str(err)
            err_trace = traceback.format_exc()
            try:
                extra_info = "user: %s" % (request.user,)
            except Exception:
                extra_info = "user: -"
            error_log = "%s\n%s\n%s" % (err_info, err_trace, extra_info)
            hash = create_hash(error_log)
            mozsvc.logger.error(hash)
            mozsvc.logger.error(error_log)
            msg = json.dumps("application error: crash id %s" % hash)
            if err.retry_after is not None:
                if err.retry_after == 0:
                    retry_after = None
                else:
                    retry_after = err.retry_after
            else:
                settings = request.registry.settings
                retry_after = settings.get("mozsvc.retry_after", 1800)

            return HTTPServiceUnavailable(body=msg, retry_after=retry_after,
                                          content_type="application/json")

    return catch_backend_errors_tween


def log_uncaught_exceptions(handler, registry):
    """Tween to log all uncaught exceptions."""

    def log_uncaught_exceptions_tween(request):
        try:
            return handler(request)
        except HTTPException:
            raise
        except Exception:
            lines = ["Uncaught exception while processing request:\n"]
            lines.append("%s %s\n" % (request.method, request.path_url))
            lines.append(safer_format_traceback(*sys.exc_info()))
            mozsvc.logger.error("".join(lines))
            raise

    return log_uncaught_exceptions_tween


def fuzz_backoff_headers(handler, registry):
    """Add some random fuzzing to the value of various backoff headers.

    This can help to avoid a "dogpile" effect where all backed-off clients
    retry at the same time and overload the server.
    """

    HEADERS = ["Retry-After", "X-Backoff", "X-Weave-Backoff"]

    def fuzz_response(response):
        for header in HEADERS:
            value = response.headers.get(header)
            if value is not None:
                # The header value is a backoff duration in seconds.  Fuzz
                # it upward by up to 5% or 5 seconds, whichever is greater.
                value = int(value)
                max_fuzz = max(int(value * 0.05), 5)
                value += random.randint(0, max_fuzz)
                response.headers[header] = str(value)

    def fuzz_backoff_headers_tween(request):
        try:
            response = handler(request)
        except HTTPException, response:
            fuzz_response(response)
            raise
        else:
            fuzz_response(response)
            return response

    return fuzz_backoff_headers_tween


def send_backoff_responses(handler, registry):
    """Send backoff/unavailable responses to a percentage of clients.

    This tween allows the server to respond to a set percentage of traffic with
    an X-Backoff header and/or a "503 Service Unavilable" response.  The two
    probabilities are controlled by config options 'mozsvc.backoff_probability'
    and 'mozsvc.unavailable_probability' respectively.  If neither option is
    set then the tween is not activated, avoiding overhead in the (hopefully!)
    common case.
    """
    settings = registry.settings
    backoff_probability = settings.get("mozsvc.backoff_probability", 0)
    unavailable_probability = settings.get("mozsvc.unavailable_probability", 0)
    retry_after = settings.get("mozsvc.retry_after", 1800)

    if backoff_probability:

        backoff_probability = float(backoff_probability)

        def add_backoff_header(response):
            if "X-Backoff" not in response.headers:
                if "X-Weave-Backoff" not in response.headers:
                    response.headers["X-Backoff"] = str(retry_after)
                    response.headers["X-Weave-Backoff"] = str(retry_after)

        def send_backoff_header_tween(request, handler=handler):
            try:
                response = handler(request)
            except HTTPException, response:
                if random.random() < backoff_probability:
                    add_backoff_header(response)
                raise
            else:
                if random.random() < backoff_probability:
                    add_backoff_header(response)
                return response

        handler = send_backoff_header_tween

    if unavailable_probability:

        unavailable_probability = float(unavailable_probability)

        def send_unavailable_response_tween(request, handler=handler):
            if random.random() < unavailable_probability:
                return HTTPServiceUnavailable(body="0",
                                              retry_after=retry_after,
                                              content_type="application/json")
            return handler(request)

        handler = send_unavailable_response_tween

    return handler


def includeme(config):
    """Include all the mozsvc tweens into the given config."""
    config.add_tween("mozsvc.tweens.catch_backend_errors")
    config.add_tween("mozsvc.tweens.log_uncaught_exceptions")
    if not config.registry.settings.get("mozsvc.dont_fuzz", False):
        config.add_tween("mozsvc.tweens.fuzz_backoff_headers")
        config.add_tween("mozsvc.tweens.send_backoff_responses")



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Contributor(s):
#   Victor Ng (vng@mozilla.com)
#   Rob Miller (rmiller@mozilla.com)
#
# ***** END LICENSE BLOCK *****
"""
Utility functions to instantiate the metrics logger from the sync
server configuration, as well as a decorator to time arbitrary
functions.
"""

import re
import json
import timeit
import logging
import functools

import pyramid.threadlocal
from pyramid.events import NewRequest


logger = logging.getLogger("mozsvc.metrics")

COMMA_SEPARATED = re.compile(r"\s*,\s*")


def initialize_request_metrics(request, defaults={}):
    """Request callback to add a "metrics" dict.

    This function should be invoked upon each new request.  It will create
    a request.metrics dict into which the application can store any runtime
    logging or metrics data, and will add response callbacks to log the
    contents of this dict once the request is complete.
    """
    # Create the request.metrics dict.
    request.metrics = defaults.copy()
    # Add in some basic information about the request
    # that should always be logged.
    request.metrics["method"] = request.method
    request.metrics["path"] = request.path_url
    request.metrics["agent"] = request.user_agent or ""
    xff = request.headers.get('X-Forwarded-For', '')
    xff = [ip for ip in COMMA_SEPARATED.split(xff) if ip]
    if request.remote_addr:
        xff.append(request.remote_addr)
    request.metrics["remoteAddressChain"] = xff
    request.metrics["request_start_time"] = timeit.default_timer()
    # Add hooks to log the metrics at the end of the request.
    request.add_response_callback(add_response_metrics)
    request.add_finished_callback(finalize_request_metrics)


def add_response_metrics(request, response):
    """Response callback to add metrics about a successfully-handled request.

    This function should be invoked when a request has been successfully
    handled and a response generated.  It will annotate the request.metrics
    dict with information about the total runtime and the resulting response.

    Note that this function is typically added to requests as a "response
    callback", which means that it will *not* be executed in case an unhandled
    exception occurs during request processing.
    """
    start_time = request.metrics.pop("request_start_time")
    request.metrics["request_time"] = timeit.default_timer() - start_time
    request.metrics["code"] = response.status_code


def finalize_request_metrics(request, message=None):
    """Finalize and log the collected request metrics.

    This function should be invoked once request handling is complete.
    It will finalize some details of the request.metrics dict and then
    emit a log line with its contents.

    By default the log line will be a simple JSON dump of all the metrics.
    This can be set to a custom message using the optional "message" argument.

    Note that this function is typically added to requests as a "finished
    callback" so that it will be invoked unconditionally at the end of
    request processing.
    """
    # If the add_response_metrics() callback did not get invoked, there
    # was probably any error in request processing.  Fill in some defaults
    # and a special status code.  The unhandled error will be logged by
    # other parts of the infrastructure.
    if "request_time" not in request.metrics:
        start_time = request.metrics.pop("request_start_time")
        request.metrics["request_time"] = timeit.default_timer() - start_time
        request.metrics["code"] = 999
    # Emit the a summary log line.
    if message is None:
        logger.info(json.dumps(request.metrics), extra=request.metrics)
    else:
        logger.info(message, extra=request.metrics)


def annotate_request(request, key, value):
    """Add or update an entry in the request.metrics dict.

    This is a helper function for storing data in the request.metrics dict.
    It provides some simple conveniences for the calling code:

        * If the request is None, then pyramid's threadlocals are used
          to find the current request object.
        * If the request has no metrics dict then it is silently ignored,
          so this is safe to call from contexts that may not metrics-enabled.
        * If the key already exists in the metrics dict, it is added to
          rather than being overwritten.

    """
    if request is None:
        request = pyramid.threadlocal.get_current_request()
    if request is not None:
        try:
            if key in request.metrics:
                request.metrics[key] += value
            else:
                request.metrics[key] = value
        except AttributeError:
            pass


class metrics_timer(object):
    """Decorator/context-manager to transparently time chunks of code.

    This class produces a timing decorator/context-manager that will place
    its result in the request.metrics dict upon completion.

    It uses pyramid threadlocals to get the current request object, which is
    a little bit icky but very convenient.  If you have the proper request
    object, you can pass it as an optional argument to the constructor.

    It can be used as a context-manager to time a chunk of code, like this:

        with metrics_timer("my.timer"):
            do_some_stuff()

    Or applied as a function decorator like this:

        @metrics_timer("my.timer")
        def do_some_stuff():
            do_more_stuff()

    """

    def __init__(self, key, request=None):
        self.key = key
        self._request = request

    def annotate_request(self, value, key=None, request=None):
        if key is None:
            key = self.key
        if request is None:
            request = self._request
        annotate_request(request, key, value)

    # When used as a context-manager, times the enclosed code.

    def __enter__(self):
        self.start_time = timeit.default_timer()
        return self

    def __exit__(self, exc_typ=None, exc_val=None, exc_tb=None):
        stop_time = timeit.default_timer()
        self.annotate_request(stop_time - self.start_time)

    # When called, applies itself as a function decorator.

    def __call__(self, func):

        @functools.wraps(func)
        def timed_func(*args, **kwds):
            # We can't use "with self" here since that stores state on
            # the object, and hence plays badly with threading or recursion.
            start_time = timeit.default_timer()
            try:
                return func(*args, **kwds)
            finally:
                stop_time = timeit.default_timer()
                self.annotate_request(stop_time - start_time)

        return timed_func


def new_request_listener(event):
    """NewRequest event-listener that adds request metrics."""
    initialize_request_metrics(event.request)


def includeme(config):
    """Include the mozsvc metrics hooks into the given config."""
    config.add_subscriber(new_request_listener, NewRequest)



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****
"""
Custom gevent-based worker class for gunicorn.

This module provides a custom GeventWorker subclass for gunicorn, with some
extra operational niceties.
"""

import os
import gc
import sys
import time
import thread
import signal
import logging
import traceback

import greenlet
import gevent.hub

from gunicorn.workers.ggevent import GeventWorker


logger = logging.getLogger("mozsvc.gunicorn_worker")

# Take references to un-monkey-patched versions of stuff we need.
# Monkey-patching will have already been done by the time we come to
# use these functions at runtime.
_real_sleep = time.sleep
_real_start_new_thread = thread.start_new_thread
_real_get_ident = thread.get_ident


# The maximum amount of time that the eventloop can be blocked
# without causing an error to be logged.
MAX_BLOCKING_TIME = float(os.environ.get("GEVENT_MAX_BLOCKING_TIME", 0.1))


# The maximum amount of memory the worker is allowed to consume, in KB.
# If it exceeds this amount it will (attempt to) gracefully terminate.
MAX_MEMORY_USAGE = os.environ.get("MOZSVC_MAX_MEMORY_USAGE", "").lower()
if MAX_MEMORY_USAGE:
    import psutil
    if MAX_MEMORY_USAGE.endswith("k"):
        MAX_MEMORY_USAGE = MAX_MEMORY_USAGE[:-1]
    MAX_MEMORY_USAGE = int(MAX_MEMORY_USAGE) * 1024
    # How frequently to check memory usage, in seconds.
    MEMORY_USAGE_CHECK_INTERVAL = 2
    # If a gc brings us back below this threshold, we can avoid termination.
    MEMORY_USAGE_RECOVERY_THRESHOLD = MAX_MEMORY_USAGE * 0.8


# The filename for dumping memory usage data.
MEMORY_DUMP_FILE = os.environ.get("MOZSVC_MEMORY_DUMP_FILE",
                                  "/tmp/mozsvc-memdump")


class MozSvcGeventWorker(GeventWorker):
    """Custom gunicorn worker with extra operational niceties.

    This is a custom gunicorn worker class, based on the standard gevent worker
    but with some extra operational- and debugging-related features:

        * a background thread that monitors execution by checking for:

            * blocking of the gevent event-loop, with tracebacks
              logged if blocking code is found.

            * overall memory usage, with forced-gc and graceful shutdown
              if memory usage goes beyond a defined limit.

        * a timeout enforced on each individual request, rather than on
          inactivity of the worker as a whole.

        * a signal handler to dump memory usage data on SIGUSR2.

    To detect eventloop blocking, the worker installs a greenlet trace
    function that increments a counter on each context switch.  A background
    (os-level) thread monitors this counter and prints a traceback if it has
    not changed within a configurable number of seconds.
    """

    def init_process(self):
        # Check if we need a background thread to monitor memory use.
        needs_monitoring_thread = False
        if MAX_MEMORY_USAGE:
            self._last_memory_check_time = time.time()
            needs_monitoring_thread = True

        # Set up a greenlet tracing hook to monitor for event-loop blockage,
        # but only if monitoring is both possible and required.
        if hasattr(greenlet, "settrace") and MAX_BLOCKING_TIME > 0:
            # Grab a reference to the gevent hub.
            # It is needed in a background thread, but is only visible from
            # the main thread, so we need to store an explicit reference to it.
            self._active_hub = gevent.hub.get_hub()
            # Set up a trace function to record each greenlet switch.
            self._active_greenlet = None
            self._greenlet_switch_counter = 0
            greenlet.settrace(self._greenlet_switch_tracer)
            self._main_thread_id = _real_get_ident()
            needs_monitoring_thread = True

        # Create a real thread to monitor out execution.
        # Since this will be a long-running daemon thread, it's OK to
        # fire-and-forget using the low-level start_new_thread function.
        if needs_monitoring_thread:
            _real_start_new_thread(self._process_monitoring_thread, ())

        # Continue to superclass initialization logic.
        # Note that this runs the main loop and never returns.
        super(MozSvcGeventWorker, self).init_process()

    def init_signals(self):
        # Leave all signals defined by the superclass in place.
        super(MozSvcGeventWorker, self).init_signals()

        # Hook up SIGUSR2 to dump memory usage information.
        # This will be useful for debugging memory leaks and the like.
        signal.signal(signal.SIGUSR2, self._dump_memory_usage)
        if hasattr(signal, "siginterrupt"):
            signal.siginterrupt(signal.SIGUSR2, False)

    def handle_request(self, *args):
        # Apply the configured 'timeout' value to each individual request.
        # Note that self.timeout is set to half the configured timeout by
        # the arbiter, so we use the value directly from the config.
        with gevent.Timeout(self.cfg.timeout):
            return super(MozSvcGeventWorker, self).handle_request(*args)

    def _greenlet_switch_tracer(self, what, (origin, target)):
        """Callback method executed on every greenlet switch.

        The worker arranges for this method to be called on every greenlet
        switch.  It keeps track of which greenlet is currently active and
        increments a counter to track how many switches have been performed.
        """
        # Increment the counter to indicate that a switch took place.
        # This will periodically be reset to zero by the monitoring thread,
        # so we don't need to worry about it growing without bound.
        self._active_greenlet = target
        self._greenlet_switch_counter += 1

    def _process_monitoring_thread(self):
        """Method run in background thread that monitors our execution.

        This method is an endless loop that gets executed in a background
        thread.  It periodically wakes up and checks:

            * whether the active greenlet has switched since last checked
            * whether memory usage is within the defined limit

        """
        # Find the minimum interval between checks.
        if MAX_MEMORY_USAGE:
            sleep_interval = MEMORY_USAGE_CHECK_INTERVAL
            if MAX_BLOCKING_TIME and MAX_BLOCKING_TIME < sleep_interval:
                sleep_interval = MAX_BLOCKING_TIME
        else:
            sleep_interval = MAX_BLOCKING_TIME
        # Run the checks in an infinite sleeping loop.
        try:
            while True:
                _real_sleep(sleep_interval)
                self._check_greenlet_blocking()
                self._check_memory_usage()
        except Exception:
            # Swallow any exceptions raised during interpreter shutdown.
            # Daemonic Thread objects have this same behaviour.
            if sys is not None:
                raise

    def _check_greenlet_blocking(self):
        if not MAX_BLOCKING_TIME:
            return
        # If there have been no greenlet switches since we last checked,
        # grab the stack trace and log an error.  The active greenlet's frame
        # is not available from the greenlet object itself, we have to look
        # up the current frame of the main thread for the traceback.
        if self._greenlet_switch_counter == 0:
            active_greenlet = self._active_greenlet
            # The hub gets a free pass, since it blocks waiting for IO.
            if active_greenlet not in (None, self._active_hub):
                frame = sys._current_frames()[self._main_thread_id]
                stack = traceback.format_stack(frame)
                err_log = ["Greenlet appears to be blocked\n"] + stack
                logger.error("".join(err_log))
        # Reset the count to zero.
        # This might race with it being incremented in the main thread,
        # but not often enough to cause a false positive.
        self._greenlet_switch_counter = 0

    def _check_memory_usage(self):
        if not MAX_MEMORY_USAGE:
            return
        elapsed = time.time() - self._last_memory_check_time
        if elapsed > MEMORY_USAGE_CHECK_INTERVAL:
            mem_usage = psutil.Process().memory_info().rss
            if mem_usage > MAX_MEMORY_USAGE:
                logger.info("memory usage %d > %d, forcing gc",
                            mem_usage, MAX_MEMORY_USAGE)
                # Try to clean it up by forcing a full collection.
                gc.collect()
                mem_usage = psutil.Process().memory_info().rss
                if mem_usage > MEMORY_USAGE_RECOVERY_THRESHOLD:
                    # Didn't clean up enough, we'll have to terminate.
                    logger.warn("memory usage %d > %d after gc, quitting",
                                mem_usage, MAX_MEMORY_USAGE)
                    self.alive = False
            self._last_memory_check_time = time.time()

    def _dump_memory_usage(self, *args):
        """Dump memory usage data to a file.

        This method writes out memory usage data for the current process into
        a timestamped file.  By default the data is written to a file named
        /tmp/mozsvc-memdump.<pid>.<timestamp> but this can be customized
        with the environment variable "MOSVC_MEMORY_DUMP_FILE".

        If the "meliae" package is not installed or if an error occurs during
        processing, then the file "mozsvc-memdump.error.<pid>.<timestamp>"
        will be written with a traceback of the error.
        """
        now = int(time.time())
        try:
            filename = "%s.%d.%d" % (MEMORY_DUMP_FILE, os.getpid(), now)
            from meliae import scanner
            scanner.dump_all_objects(filename)
        except Exception:
            filename = "%s.error.%d.%d" % (MEMORY_DUMP_FILE, os.getpid(), now)
            with open(filename, "w") as f:
                f.write("ERROR DUMPING MEMORY USAGE\n\n")
                traceback.print_exc(file=f)




import os
import json
import logging
import unittest2

from testfixtures import LogCapture

from mozsvc.util import JsonLogFormatter


class TestJsonLogFormatter(unittest2.TestCase):

    def setUp(self):
        self.handler = LogCapture()
        self.formatter = JsonLogFormatter()

    def tearDown(self):
        self.handler.uninstall()

    def test_basic_operation(self):
        logging.debug("simple test")
        self.assertEquals(len(self.handler.records), 1)
        details = json.loads(self.formatter.format(self.handler.records[0]))
        self.assertEquals(details["message"], "simple test")
        self.assertEquals(details["name"], "root")
        self.assertEquals(details["pid"], os.getpid())
        self.assertEquals(details["op"], "root")
        self.assertEquals(details["v"], 1)
        self.assertTrue("time" in details)

    def test_custom_paramters(self):
        logger = logging.getLogger("mozsvc.test.test_logging")
        logger.warn("custom test %s", "one", extra={
            "more": "stuff",
            "op": "mytest",
        })
        self.assertEquals(len(self.handler.records), 1)
        details = json.loads(self.formatter.format(self.handler.records[0]))
        self.assertEquals(details["message"], "custom test one")
        self.assertEquals(details["name"], "mozsvc.test.test_logging")
        self.assertEquals(details["op"], "mytest")
        self.assertEquals(details["more"], "stuff")

    def test_logging_error_tracebacks(self):
        try:
            raise ValueError("\n")
        except Exception:
            logging.exception("there was an error")
        self.assertEquals(len(self.handler.records), 1)
        details = json.loads(self.formatter.format(self.handler.records[0]))
        self.assertEquals(details["message"], "there was an error")
        self.assertEquals(details["error"], "ValueError('\\n',)")
        tblines = details["traceback"].strip().split("\n")
        self.assertEquals(tblines[-1], details["error"])
        self.assertEquals(tblines[-2], "<type 'exceptions.ValueError'>")



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import os

from konfig import Config, SettingsDict

from pyramid.config import Configurator


def load_into_settings(filename, settings):
    """Load config file contents into a Pyramid settings dict.

    This is a helper function for initialising a Pyramid settings dict from
    a config file.  It flattens the config file sections into dotted settings
    names and updates the given dictionary in place.

    You would typically use this when constructing a Pyramid Configurator
    object, like so::

        def main(global_config, **settings):
            config_file = global_config['__file__']
            load_info_settings(config_file, settings)
            config = Configurator(settings=settings)

    """
    filename = os.path.expandvars(os.path.expanduser(filename))
    filename = os.path.abspath(os.path.normpath(filename))
    config = Config(filename)

    # Put values from the config file into the pyramid settings dict.
    for section in config.sections():
        setting_prefix = section.replace(":", ".")
        for name, value in config.get_map(section).iteritems():
            settings[setting_prefix + "." + name] = value

    # Store a reference to the Config object itself for later retrieval.
    settings['config'] = config
    return config


def get_configurator(global_config, **settings):
    """Create a pyramid Configurator and populate it with sensible defaults.

    This function is a helper to create and pre-populate a Configurator
    object using the given paste-deploy settings dicts.  It uses the
    mozsvc.config module to flatten the config paste-deploy config file
    into the settings dict so that non-mozsvc pyramid apps can read values
    from it easily.
    """
    # Populate a SettingsDict with settings from the deployment file.
    settings = SettingsDict(settings)
    config_file = global_config.get('__file__')
    if config_file is not None:
        load_into_settings(config_file, settings)
    # Update with default pyramid settings, and then insert for all to use.
    config = Configurator(settings={})
    settings.setdefaults(config.registry.settings)
    config.registry.settings = settings
    return config
